#include <iostream>
#include <string>

// подключили хедер библиотеки Boost.regex откуда-то из стандартных путей
#include <boost/regex.hpp>

int main(int argc, char ** argv) {
  // описываем переменную r - регулярное выражение
  // инициализируем это выражение из строки, описывающей формат в особом синтаксисе: ECMAScript syntax (см., например, cplusplus.com)
  // здесь этот формат говорит:
  // * сначала идёт буква c
  // * потом любое ненулевое число букв a
  // * потом любое ненулевое число букв b
  // * потом буква d
  // * потом число в диапазоне от 0 до 9
  // * при этом если строка подошла под выражение, то помимо прочего я хочу в отдельных подстроках получить
  // ** подслово с первой буквы a по последнюю букву b (это внешние круглые скобки)
  // ** подслово из всех букв a (это левые внутренние круглые скобки)
  // ** подслово из всех букв b (это правые внутренние круглые скобки)
  boost::regex r("c((a+)(b+))d[0-9]");
  
  // если выражение подойдёт под формат r, то нужно где-то хранить результат - в числе прочего, те подстроки, которые я хочу получить согласно формату
  // если я буду проверять выражение на объекте типа std::string, то результат хранится в smatch
  // в общем случае см. класс match_results и его специализации, в числе которых smatch
  boost::smatch m;
  
  // соорудил строку, которую буду матчить выражением (проверять, подходит ли под формат, и вычленять подстроки)
  std::string s = "caaabbbd5";
  
  // матчим!
  // строку s по формату r, и если заматчится, то результат запишется в m
  boost::regex_match(s, m, r);
  
  // если строка заматчилась успешно, то в m (как в контейнере vector) записан массив структур, содержащих информацию для заматченных подстрок
  // у нас это структуры для: всей заматченной строки; подслова от первой a до последней b; подслова из всех a; подслова из всех b
  // в частности, если хочется узнать, что же за строка заматчилась, то это делается методом str() для структуры
  // например, m[0].str() - это вся заматченная строка, а m[1].str() - это подстрока aaabbb
  //
  // про конструкцию "for(const auto & el : m)" смотри комментарии в CMakefiles.txt
  for(const auto & el : m)
    std::cout << el.str() << std::endl;
  return 0;
}
